//
//  AppDelegate.m
//  Dialectic Framework Tester
//
//  Created by Jon Nathan, JNSoftware LLC.
//  Copyright 2015 JNSoftware LLC. All rights reserved.
//

#import "AppDelegate.h"

static NSUserDefaults *standardUserDefaults = nil;

@implementation AppDelegate

@synthesize window, activateDialecticButton, currentLocationPopUpButton, trackedCalls, sharedDialecticController;

+ (void)initialize {
	static BOOL isInitialized = NO;
	if (!isInitialized) {
		standardUserDefaults = [NSUserDefaults standardUserDefaults];
		NSDictionary *appDefaults = [NSDictionary dictionaryWithObjectsAndKeys:
									 @"", DialecticContactNumberKey, 
									 @"", DialecticContactNameKey, 
									 [NSNumber numberWithBool:NO], DialecticLiteralKey, 
									 @"", DialecticLocationKey, 
									 @"", DialecticIdentifierKey, 
									 @"", @"contactNumberNotify", 
									 @"", @"contactNameNotify", 
									 @"", @"deviceNotify", 
									 @"", @"noteNotify", 
									 [NSNumber numberWithBool:NO], @"flaggedNotify", 
									 @"", @"identifierNotify", 
									 @"", @"largeTypeMesssage", 
									 nil];
		[standardUserDefaults registerDefaults:appDefaults];
		[standardUserDefaults synchronize];
		isInitialized = YES;
	}
}

- (void)dealloc {
	self.trackedCalls = nil;
	self.sharedDialecticController = nil;
	[super dealloc];
}

- (void)notificationReceived:(NSNotification *)notification {
	//requires that Dialectic be set to broadcast outgoing and/or incoming calls respectively
	NSDictionary *userInfo = [notification userInfo];
	NSString *identifier = [userInfo objectForKey:DialecticIdentifierKey];
	if ([trackedCalls indexOfObject:identifier] == NSNotFound) return; // not interested in calls not initiated by us; remove this to be notified of all incoming & outgoing calls

	if ([[notification name] isEqualToString:DialecticIncomingCallDidStartNotification]) {
		// do something when an incoming call is started...
		NSLog(@"Our incoming call started: %@", userInfo);

	} else if ([[notification name] isEqualToString:DialecticIncomingCallDidEndNotification]) {
		// do something when an incoming call is ended...
		NSLog(@"Our incoming call ended: %@", userInfo);
		[trackedCalls removeObject:identifier];

	} else if ([[notification name] isEqualToString:DialecticOutgoingCallDidStartNotification]) {
		// do something when an outgoing call is started; userInfo will include a key indicating success or failure of the call...
		NSLog(@"Our outgoing call started: %@", userInfo);

	} else if ([[notification name] isEqualToString:DialecticOutgoingCallDidEndNotification]) {
		// do something when an outgoing call is ended...
		NSLog(@"Our outgoing call ended: %@", userInfo);
		[trackedCalls removeObject:identifier];
	}
}

- (void)dialecticIncomingCallDidStart:(NSNotification *)notification {
	[self notificationReceived:notification];
}

- (void)dialecticIncomingCallDidEnd:(NSNotification *)notification {
	[self notificationReceived:notification];
}

- (void)dialecticOutgoingCallDidStart:(NSNotification *)notification {
	[self notificationReceived:notification];
}

- (void)dialecticOutgoingCallDidEnd:(NSNotification *)notification {
	[self notificationReceived:notification];
}

- (void)commonInit {
	//ensures the sharedDialecticController is initialized before it's called elsewhere
	if (!sharedDialecticController) {
		self.sharedDialecticController = [DialecticController sharedDialecticController];
		sharedDialecticController.delegate = self;
	}
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification {
	[self commonInit];
	self.trackedCalls = [NSMutableArray array];
	activateDialecticButton.image = [sharedDialecticController dialecticIcon];
	[currentLocationPopUpButton selectItemWithTitle:[sharedDialecticController currentDialecticLocationName]];
	NSString *location = [standardUserDefaults objectForKey:DialecticLocationKey];
	if ([location isEqualToString:@""]) [standardUserDefaults setObject:[sharedDialecticController currentDialecticLocationName] forKey:DialecticLocationKey];
	if ([standardUserDefaults objectForKey:[NSString stringWithFormat:@"NSWindow Frame %@", [window frameAutosaveName]]] == nil) [window center];
	[window makeKeyAndOrderFront:nil];
	NSString *message = nil;
	NSString *informativeText = nil;
	BOOL showWebpageButton = NO;
	if (![sharedDialecticController isDialecticInstalled]) {
		message = @"Dialectic Not Installed";
		informativeText = @"Dialectic is not installed on this Mac. You can download the latest version of Dialectic from the product webpage.";
		showWebpageButton = YES;
	} else if (![sharedDialecticController isDialecticRunning]) {
		message = @"Dialectic Not Running";
		informativeText = @"Dialectic is not Running on this Mac. This means the popup buttons won't show the Dialectic locations.\n\nYou can launch Dialectic by clicking the Dialectic icon in the main window -- it's a button! After Dialectic is launched, click the button again to load the location popups.";
	}
	if (informativeText) {
		NSBeep();
		NSAlert *alert = [[[NSAlert alloc] init] autorelease];
		((NSWindow *)alert.window).title = [[NSBundle mainBundle] objectForInfoDictionaryKey:(id)kCFBundleNameKey];
		alert.alertStyle = NSCriticalAlertStyle;
		alert.messageText = message;
		alert.informativeText = informativeText;
		[alert addButtonWithTitle:NSLocalizedString(@"OK", @"")];
		if (showWebpageButton) [alert addButtonWithTitle:NSLocalizedString(@"Goto Webpage", @"")];
		NSModalResponse response = [alert runModal];
		if (response == NSAlertSecondButtonReturn) [[NSWorkspace sharedWorkspace] openURL:[sharedDialecticController dialecticProductWebpageURL]];
	}
}

- (void)awakeFromNib {
	[self commonInit];
}

- (NSArray *)dialecticLocations {
	[self commonInit];
	return [sharedDialecticController dialecticLocationNames];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)application {
	return YES;
}

- (void)applicationWillTerminate:(NSNotification *)notification {
}

- (IBAction)activateDialectic:(id)sender {
	[self willChangeValueForKey:@"dialecticLocations"];
	[sharedDialecticController activateDialectic];
	[self didChangeValueForKey:@"dialecticLocations"];
	[currentLocationPopUpButton selectItemWithTitle:[sharedDialecticController currentDialecticLocationName]];
}

- (IBAction)setCurrentDialecticLocation:(id)sender {
	[sharedDialecticController setCurrentDialecticLocation:[sender title]];
}

- (IBAction)dial:(id)sender {
	NSString *contactNumber = [standardUserDefaults objectForKey:DialecticContactNumberKey];
	if (!contactNumber || ([contactNumber length] == 0)) {
		NSBeep();
		NSAlert *alert = [[[NSAlert alloc] init] autorelease];
		((NSWindow *)alert.window).title = [[NSBundle mainBundle] objectForInfoDictionaryKey:(id)kCFBundleNameKey];
		alert.alertStyle = NSCriticalAlertStyle;
		alert.messageText = NSLocalizedString(@"No Number Provided", @"");
		alert.informativeText = NSLocalizedString(@"You must specify a phone number before dialing", @"");
		[alert addButtonWithTitle:NSLocalizedString(@"OK", @"")];
		[alert runModal];
		return;
	}
	NSString *contactName = [standardUserDefaults objectForKey:DialecticContactNameKey];
	NSNumber *literal = [standardUserDefaults objectForKey:DialecticLiteralKey];
	NSString *location = [standardUserDefaults objectForKey:DialecticLocationKey];
	NSString *identifier = [standardUserDefaults objectForKey:DialecticIdentifierKey];
	NSMutableDictionary *callData = [NSMutableDictionary dictionary];
	[callData setObject:contactNumber forKey:DialecticContactNumberKey]; //required; a phone number; [string]
	if (contactName) [callData setObject:contactName forKey:DialecticContactNameKey]; //optional; a contact name, can be blank; [string]
	if (literal) [callData setObject:literal forKey:DialecticLiteralKey]; //optional; if included, must be boolean NSNumber to indicate to Dialectic that the number should be dialed as formatted and not processed by the Dialectic dialing rules; [number]
	if (location) [callData setObject:location forKey:DialecticLocationNameKey]; //optional; the name of an existing Dialectic location to use for this call, can be blank, if omitted or blank, the current location will be used; [string]
	if (identifier) [callData setObject:identifier forKey:DialecticIdentifierKey]; //optional; any string to identify the call from a later call back, can be blank, if omitted or blank, the a UUID will be assigned to the call; [string]
	if (identifier && ([identifier length] > 0)) [trackedCalls addObject:identifier];
	NSError *error = nil;
	BOOL success = [sharedDialecticController dialDialectic:[NSDictionary dictionaryWithDictionary:callData] error:&error];
	if (error) NSLog(@"dial error: %@", [error localizedDescription]);
	NSLog(@"dial success: %d", success);
}

- (IBAction)notify:(id)sender {
	NSString *contactNumber = [standardUserDefaults objectForKey:@"contactNumberNotify"];
	NSString *contactName = [standardUserDefaults objectForKey:@"contactNameNotify"];
	NSString *device = [standardUserDefaults objectForKey:@"deviceNotify"];
	NSString *note = [standardUserDefaults objectForKey:@"noteNotify"];
	NSNumber *flagged = [standardUserDefaults objectForKey:@"flaggedNotify"];
	NSString *identifier = [standardUserDefaults objectForKey:@"identifierNotify"];
	NSMutableDictionary *callData = [NSMutableDictionary dictionary];
	if (contactNumber) [callData setObject:contactNumber forKey:DialecticContactNumberKey]; //optional; a phone number, can be blank; [string]
	if (contactName) [callData setObject:contactName forKey:DialecticContactNameKey]; //optional; a contact name, can be blank; [string]
	if (device) [callData setObject:device forKey:DialecticDeviceKey]; //optional; the name of the device (i.e., the phone or PBX), can be blank; [string]
	if (note) [callData setObject:note forKey:DialecticNoteKey]; //optional; a note for the call log, can be blank; [string]
	if (flagged) [callData setObject:flagged forKey:DialecticFlaggedKey]; //optional; if included, must be boolean NSNumber to flag the call log entry for this call; [number]
	if (identifier) [callData setObject:identifier forKey:DialecticIdentifierKey]; //optional; any string to identify the call from a later call back, can be blank, if omitted or blank, the a UUID will be assigned to the call; [string]
	if (identifier && ([identifier length] > 0)) [trackedCalls addObject:identifier];
	NSError *error = nil;
	BOOL success = [sharedDialecticController notifyDialecticOfIncomingCall:[NSDictionary dictionaryWithDictionary:callData] error:&error];
	if (error) NSLog(@"notify error: %@", [error localizedDescription]);
	NSLog(@"notify success: %d", success);
}

- (NSArray *)logItems {
	return [NSArray arrayWithObjects:
			@"Log Something", 
			@"Dialectic Bundle Identifier", 
			@"Dialectic Path", 
			@"Dialectic Version", 
			@"Is Dialectic Installed?", 
			@"Is Dialectic Running?", 
			@"Call Log Entries", 
			@"Quick Dial Numbers", 
			@"Recently Dialed Numbers", 
			nil];
}

- (IBAction)logSomething:sender {
	if ([[sender titleOfSelectedItem] isEqualToString:[[self logItems] objectAtIndex:1]]) {
		NSLog(@"dialecticBundleIdentifier: %@", [sharedDialecticController dialecticBundleIdentifier]);
	} else if ([[sender titleOfSelectedItem] isEqualToString:[[self logItems] objectAtIndex:2]]) {
		NSLog(@"dialecticPath: %@", [sharedDialecticController dialecticPath]);
	} else if ([[sender titleOfSelectedItem] isEqualToString:[[self logItems] objectAtIndex:3]]) {
		NSLog(@"dialecticVersion: %@", [sharedDialecticController dialecticVersion]);
	} else if ([[sender titleOfSelectedItem] isEqualToString:[[self logItems] objectAtIndex:4]]) {
		NSLog(@"isDialecticInstalled: %d", [sharedDialecticController isDialecticInstalled]);
	} else if ([[sender titleOfSelectedItem] isEqualToString:[[self logItems] objectAtIndex:5]]) {
		NSLog(@"isDialecticRunning: %d", [sharedDialecticController isDialecticRunning]);
	} else if ([[sender titleOfSelectedItem] isEqualToString:[[self logItems] objectAtIndex:6]]) {
		NSLog(@"callLogEntries: %@", [sharedDialecticController dialecticCallLogEntries]);
	} else if ([[sender titleOfSelectedItem] isEqualToString:[[self logItems] objectAtIndex:7]]) {
		NSLog(@"quickDialNumbers, %@", [sharedDialecticController dialecticQuickDialNumbers]);
	} else if ([[sender titleOfSelectedItem] isEqualToString:[[self logItems] objectAtIndex:8]]) {
		NSLog(@"recentlyDialNumbers, %@", [sharedDialecticController dialecticRecentlyDialedNumbers]);
	}
}

- (IBAction)showLargeType:(id)sender {
	NSString *largeTypeMesssage = [standardUserDefaults objectForKey:@"largeTypeMesssage"];
	if (!largeTypeMesssage || ([largeTypeMesssage length] == 0)) {
		NSBeep();
		NSAlert *alert = [[[NSAlert alloc] init] autorelease];
		((NSWindow *)alert.window).title = [[NSBundle mainBundle] objectForInfoDictionaryKey:(id)kCFBundleNameKey];
		alert.alertStyle = NSCriticalAlertStyle;
		alert.messageText = NSLocalizedString(@"No Message Provided", @"");
		alert.informativeText = NSLocalizedString(@"You must specify a message to show.", @"");
		[alert addButtonWithTitle:NSLocalizedString(@"OK", @"")];
		[alert runModal];
		return;
	}
	[sharedDialecticController showLargeType:largeTypeMesssage];
}

@end
